@@ -3,25 +3,24 @@
 # Copyright (C) 2008 Drew Hess <dhess@bothan.net>
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as
-# published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Affero General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Affero General Public
-# License along with this program.  If not, see
-# <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Parse and import U.S. Senate LD-1/LD-2 XML documents."""
 
 import xml.dom.pulldom
 
 
-VERSION = '0.9'
+VERSION = '0.10'
 
 
 # Attribute parsers.
@@ -295,8 +294,39 @@ def _parse_issues(elt):
     return _parse_list(elt, 'issues', _parse_issue)
 
 
+_foreign_entity_attrs = [('ForeignEntityContribution', 'contribution', _amount),
+                         ('ForeignEntityCountry', 'country', _optional),
+                         ('ForeignEntityName', 'name', _identity),
+                         ('ForeignEntityOwnershipPercentage',
+                              'ownership_percentage', _amount),
+                         ('ForeignEntityPPBcountry', 'ppb_country', _optional),
+                         ('ForeignEntityStatus', 'status', _status)]
+
+
+def _parse_foreign_entity(elt):
+    """Parse a (foreign) Entity DOM element.
+
+    elt - The (foreign) Entity DOM element.
+
+    Returns a pair whose first item is the string 'foreign_entity' and
+    whose second item is the dictionary of parsed attributes.
+
+    """
+    return _parse_element(elt, 'foreign_entity', _foreign_entity_attrs)
+
+
 def _parse_foreign_entities(elt):
-    return ('foreign_entities', list())
+    """Parse a ForeignEntities DOM element.
+
+    elt - The ForeignEntities DOM element.
+
+    Returns a pair whose first item is the string 'foreign_entities'
+    and whose second item is a list of parsed (foreign) Entity DOM
+    elements, one for each Entity sub-element of this ForeignEntities
+    element.
+
+    """
+    return _parse_list(elt, 'foreign_entities', _parse_foreign_entity)
 
 
 # The affiliated org PPB country attribute name is spelled,
@@ -392,38 +422,41 @@ def parse_filings(doc):
 # Code to import parsed records into the database.
 
 _where_stmt = {'client':
-                   'client WHERE ' \
-                   'country=:country AND ' \
-                   'senate_id=:senate_id AND '\
-                   'name=:name AND ' \
-                   'ppb_country=:ppb_country AND ' \
-                   'state=:state AND ' \
-                   'ppb_state=:ppb_state AND ' \
-                   'status=:status AND ' \
-                   'description=:description AND ' \
-                   'state_or_local_gov=:state_or_local_gov AND ' \
-                   'contact_name=:contact_name',
+                   'client WHERE '
+                   'country=:country AND '
+                   'name=:name AND '
+                   'ppb_country=:ppb_country AND '
+                   'state=:state AND '
+                   'ppb_state=:ppb_state AND '
+                   'state_or_local_gov=:state_or_local_gov',
                'registrant':
-                   'registrant WHERE ' \
-                   'address=:address AND ' \
-                   'description=:description AND ' \
-                   'country=:country AND ' \
-                   'senate_id=:senate_id AND ' \
-                   'name=:name AND ' \
+                   'registrant WHERE '
+                   'country=:country AND '
+                   'senate_id=:senate_id AND '
+                   'name=:name AND '
                    'ppb_country=:ppb_country',
                'lobbyist':
-                   'lobbyist WHERE ' \
-                   'name=:name AND ' \
-                   'status=:status AND ' \
-                   'indicator=:indicator AND ' \
+                   'lobbyist WHERE '
+                   'name=:name AND '
+                   'indicator=:indicator AND '
                    'official_position=:official_position',
                'affiliated_org':
-                   'affiliated_org WHERE ' \
-                   'name=:name AND ' \
-                   'country=:country AND ' \
+                   'affiliated_org WHERE '
+                   'name=:name AND '
+                   'country=:country AND '
+                   'ppb_country=:ppb_country',
+               'foreign_entity':
+                   'foreign_entity WHERE '
+                   'name=:name AND '
+                   'country=:country AND '
                    'ppb_country=:ppb_country'}
 
 
+def _filing_db_key(filing):
+    """Return a filing's database key."""
+    return filing['id']
+
+
 def _rowid(table, tomatch, cur):
     """Find a match in a database table and return its rowid.
 
@@ -465,32 +498,43 @@ def _client_rowid(client, cur):
     return _rowid('client', client, cur)
 
 
-def _insert_client(client, cur):
-    """Insert a client into the database.
+def _import_client(client, filing, cur):
+    """Import a client into the database.
 
-    Returns the row ID of the inserted client.
+    Returns nothing.
 
-    As a side effect, this function also inserts rows into the
-    'country', 'state' and 'org' tables.
+    Side-effects: may insert rows into the 'client', 'country',
+    'state', 'org' and 'filing_client' tables.
 
     client - The parsed client dictionary.
 
+    filing - The parsed filing dictionary with which the client is
+    associated.
+
     cur - The DB API 2.0-compliant database cursor.
 
     """
-    # Note - client status is pre-inserted into client_status table.
-    for key in ['country', 'ppb_country']:
-        cur.execute('INSERT INTO country VALUES(?)', [client[key]])
-    for key in ['state', 'ppb_state']:
-        cur.execute('INSERT INTO state VALUES(?)', [client[key]])
-    cur.execute('INSERT INTO person VALUES(?)', [client['contact_name']])
-    cur.execute('INSERT INTO org VALUES(?)', [client['name']])
-    cur.execute('INSERT INTO client VALUES(NULL, \
-                   :country, :senate_id, :name, :ppb_country, \
-                   :state, :ppb_state, :status, :description, \
-                   :state_or_local_gov, :contact_name)',
-                client)
-    return cur.lastrowid
+    db_key = _client_rowid(client, cur)
+    if db_key is None:
+        # Note - client status is pre-inserted into client_status table.
+        for key in ['country', 'ppb_country']:
+            cur.execute('INSERT INTO country VALUES(?)', [client[key]])
+        for key in ['state', 'ppb_state']:
+            cur.execute('INSERT INTO state VALUES(?)', [client[key]])
+        cur.execute('INSERT INTO person VALUES(?)', [client['contact_name']])
+        cur.execute('INSERT INTO org VALUES(?)', [client['name']])
+        cur.execute('INSERT INTO client VALUES(NULL, '
+                    ':country, :name, :ppb_country, :state, :ppb_state, '
+                    ':state_or_local_gov)',
+                    client)
+        db_key = cur.lastrowid
+    cur.execute('INSERT INTO filing_client VALUES(?, ?, ?, ?, ?, ?)',
+                [_filing_db_key(filing),
+                 db_key,
+                 client['senate_id'],
+                 client['status'],
+                 client['contact_name'],
+                 client['description']])
 
 
 def _registrant_rowid(reg, cur):
@@ -507,30 +551,39 @@ def _registrant_rowid(reg, cur):
     return _rowid('registrant', reg, cur)
 
 
-def _insert_registrant(reg, cur):
-    """Insert a registrant into the database.
+def _import_registrant(reg, filing, cur):
+    """Import a registrant into the database.
 
-    Returns the row ID of the inserted registrant.
+    Returns nothing.
 
-    As a side effect, this function also inserts rows into the country
-    and org tables.
+    Side-effects: may insert rows into the 'registrant', 'country',
+    'org' and 'filing_registrant' tables.
 
     reg - The parsed registrant dictionary.
 
+    filing - The parsed filing dictionary with which the registrant is
+    associated.
+
     cur - The DB API 2.0-compliant database cursor.
 
     """
-    cur.execute('INSERT INTO country VALUES(?)',
-                [reg['country']])
-    cur.execute('INSERT INTO country VALUES(?)',
-                [reg['ppb_country']])
-    cur.execute('INSERT INTO org VALUES(?)',
-                [reg['name']])
-    cur.execute('INSERT INTO registrant VALUES(NULL, \
-                           :address, :description, :country, :senate_id, \
-                           :name, :ppb_country)',
-                reg)
-    return cur.lastrowid
+    db_key = _registrant_rowid(reg, cur)
+    if db_key is None:
+        cur.execute('INSERT INTO country VALUES(?)',
+                    [reg['country']])
+        cur.execute('INSERT INTO country VALUES(?)',
+                    [reg['ppb_country']])
+        cur.execute('INSERT INTO org VALUES(?)',
+                    [reg['name']])
+        cur.execute('INSERT INTO registrant VALUES(NULL, '
+                    ':country, :senate_id, :name, :ppb_country)',
+                    reg)
+        db_key = cur.lastrowid
+    cur.execute('INSERT INTO filing_registrant VALUES(?, ?, ?, ?)',
+                [_filing_db_key(filing),
+                 db_key,
+                 reg['address'],
+                 reg['description']])
 
 
 def _lobbyist_rowid(lobbyist, cur):
@@ -547,59 +600,79 @@ def _lobbyist_rowid(lobbyist, cur):
     return _rowid('lobbyist', lobbyist, cur)
 
 
-def _insert_lobbyist(lobbyist, cur):
-    """Insert a lobbyist into the database.
+def _import_lobbyist(lobbyist, filing, cur):
+    """Import a lobbyist into the database.
 
-    Returns the row ID of the inserted lobbyist.
+    Returns nothing.
 
-    As a side effect, this function also inserts rows into the
-    'person' table.
+    Side-effects: may insert rows into the 'lobbyist', 'person' and
+    'filing_lobbyists' tables.
 
     lobbyist - The parsed lobbyist dictionary.
 
+    filing - The parsed filing dictionary with which the lobbyist is
+    associated.
+
     cur - The DB API 2.0-compliant database cursor.
 
     """
-    # Note - lobbyist status and indicator are pre-inserted into the
-    # lobbyist_status and lobbyist_indicator tables.
-    cur.execute('INSERT INTO person VALUES(?)', [lobbyist['name']])
-    cur.execute('INSERT INTO lobbyist VALUES(NULL, \
-                   :name, :status, :indicator, :official_position)',
-                lobbyist)
-    return cur.lastrowid
+    db_key = _lobbyist_rowid(lobbyist, cur)
+    if db_key is None:
+        # Note - lobbyist status and indicator are pre-inserted into the
+        # lobbyist_status and lobbyist_indicator tables.
+        cur.execute('INSERT INTO person VALUES(?)', [lobbyist['name']])
+        cur.execute('INSERT INTO lobbyist VALUES(NULL, '
+                    ':name, :indicator, :official_position)',
+                    lobbyist)
+        db_key = cur.lastrowid
+    cur.execute('INSERT INTO filing_lobbyists VALUES(?, ?, ?)',
+                [_filing_db_key(filing), db_key, lobbyist['status']])
 
 
-def _insert_govt_entity(entity, cur):
-    """Insert a government entity into the database.
+def _import_govt_entity(entity, filing, cur):
+    """Import a government entity into the database.
 
-    Returns the key (NOT the rowid!) of the inserted entry.
+    Returns nothing.
+
+    Side-effects: may insert rows into the 'govt_entity' and
+    'filing_govt_entities' tables.
 
     entity - The parsed government entity dictionary.
 
+    filing - The parsed filing dictionary with which the government
+    entity is associated.
+
     cur - The DB API 2.0-compliant database cursor.
 
     """
+    db_key = entity['name']
     cur.execute('INSERT INTO govt_entity VALUES(:name)', entity)
-    return entity['name']
+    cur.execute('INSERT INTO filing_govt_entities VALUES(?, ?)',
+                [_filing_db_key(filing), db_key])
 
 
-def _insert_issue(issue, cur):
-    """Insert an issue into the database.
+def _import_issue(issue, filing, cur):
+    """Import an issue into the database.
 
-    Returns the row ID of the inserted issue.
+    Returns nothing.
 
-    As a side effect, this function also inserts rows into the
-    'issue_code' table.
+    Side-effects: may insert rows into the 'issue', 'issue_code' and
+    'filing_issues' tables.
 
     issue - The parsed issue dictionary.
 
+    filing - The parsed filing dictionary with which the issue is
+    associated.
+
     cur - The DB API 2.0-compliant database cursor.
 
     """
     cur.execute('INSERT INTO issue_code VALUES(?)', [issue['code']])
     cur.execute('INSERT INTO issue VALUES(NULL, :code, :specific_issue)',
                 issue)
-    return cur.lastrowid
+    db_key = cur.lastrowid
+    cur.execute('INSERT INTO filing_issues VALUES(?, ?)',
+                [_filing_db_key(filing), db_key])
 
 
 def _affiliated_org_rowid(org, cur):
@@ -616,175 +689,144 @@ def _affiliated_org_rowid(org, cur):
     return _rowid('affiliated_org', org, cur)
 
 
-def _insert_affiliated_org(org, cur):
-    """Insert an affiliated org into the database.
+def _import_affiliated_org(org, filing, cur):
+    """Import an affiliated org into the database.
 
-    Returns the row ID of the inserted org.
-
-    As a side effect, this function also inserts rows into the
-    'country' and 'org' tables.
+    Returns nothing.
+    
+    Side-effects: may insert rows into the 'affiliated_org',
+    'country', 'org', 'filing_affiliated_orgs', and 'url' tables.
 
     org - The parsed org dictionary.
 
+    filing - The parsed filing dictionary with which the org is
+    associated.
+
     cur - The DB API 2.0-compliant database cursor.
 
     """
-    for key in ['country', 'ppb_country']:
-        cur.execute('INSERT INTO country VALUES(?)', [org[key]])
-    cur.execute('INSERT INTO org VALUES(?)', [org['name']])
-    cur.execute('INSERT INTO affiliated_org VALUES(NULL, \
-                   :name, :country, :ppb_country)',
-                org)
-    return cur.lastrowid
+    db_key = _affiliated_org_rowid(org, cur)
+    if db_key is None:
+        for key in ['country', 'ppb_country']:
+            cur.execute('INSERT INTO country VALUES(?)', [org[key]])
+        cur.execute('INSERT INTO org VALUES(?)', [org['name']])
+        cur.execute('INSERT INTO affiliated_org VALUES(NULL, '
+                    ':name, :country, :ppb_country)',
+                    org)
+        db_key = cur.lastrowid
+    url = filing['affiliated_orgs_url']
+    cur.execute('INSERT INTO url VALUES(?)', [url])
+    cur.execute('INSERT INTO filing_affiliated_orgs VALUES(?, ?, ?)',
+                [_filing_db_key(filing), db_key, url])
 
 
-def _insert_filing(filing, cur):
-    """Insert a filing and its relationships into the database.
+def _foreign_entity_rowid(entity, cur):
+    """Find a foreign entity in the database.
 
-    Returns the row ID of the inserted filing.
+    Returns the row ID of the matching entity, or None if there is no
+    match.
 
+    entity - The parsed foreign entity dictionary.
+
+    cur - The DB API 2.0-compliant database cursor.
+
+    """
+    return _rowid('foreign_entity', entity, cur)
+
+
+def _import_foreign_entity(entity, filing, cur):
+    """Import a foreign entity into the database.
+
+    Returns nothing.
+
+    Side-effects: may insert rows into the 'foreign_entity',
+    'country', 'org' and 'filing_foreign_entities' tables.
+
+    entity - The parsed foreign entity dictionary.
+
+    filing - The parsed filing dictionary with which the foreign
+    entity is associated.
+
+    cur - The DB API 2.0-compliant database cursor.
+
+    """
+    db_key = _foreign_entity_rowid(entity, cur)
+    if db_key is None:
+        for key in ['country', 'ppb_country']:
+            cur.execute('INSERT INTO country VALUES(?)', [entity[key]])
+        cur.execute('INSERT INTO org VALUES(?)', [entity['name']])
+        cur.execute('INSERT INTO foreign_entity VALUES(NULL, '
+                    ':name, :country, :ppb_country)',
+                    entity)
+        db_key = cur.lastrowid
+    cur.execute('INSERT INTO filing_foreign_entities VALUES(?, ?, ?, ?, ?)',
+                [_filing_db_key(filing),
+                 db_key,
+                 entity['contribution'],
+                 entity['ownership_percentage'],
+                 entity['status']])
+                 
+
+def _import_filing(filing, cur):
+    """Import a filing into the database.
+
+    Returns nothing.
+
+    Side-effects: inserts a row into the 'filing' table.
+    
     filing - The parsed filing dictionary.
 
     cur - The DB API 2.0-compliant database cursor.
 
     """
-    # Insert the filing first, then its relationships.
-    cur.execute('INSERT INTO filing VALUES(\
-                       :id, :type, :year, :period, :filing_date, :amount, \
-                       :registrant, :client)',
+    # The affiliated orgs URL is a special case. It's associated with
+    # each affiliated org in the record, so it's handled by the
+    # affiliated org importer, and we skip it here.
+    cur.execute('INSERT INTO filing VALUES('
+                ':id, :type, :year, :period, :filing_date, :amount)',
                 filing)
-    filing_rowid = cur.lastrowid
-    # The affiliated orgs URL is a special case. It should really be
-    # an attribute of the AffiliatedOrgs element, not the Filing
-    # element.
-    cur.execute('INSERT INTO url VALUES(:affiliated_orgs_url)', filing)
-    for id in filing['lobbyists']:
-        cur.execute('INSERT INTO filing_lobbyists VALUES(?, ?)',
-                    [filing['id'], id])
-    for id in filing['govt_entities']:
-        cur.execute('INSERT INTO filing_govt_entities VALUES(?, ?)',
-                    [filing['id'], id])
-    for id in filing['issues']:
-        cur.execute('INSERT INTO filing_issues VALUES(?, ?)',
-                    [filing['id'], id])
-    for id in filing['affiliated_orgs']:
-        cur.execute('INSERT INTO filing_affiliated_orgs VALUES(?, ?)',
-                    [filing['id'], id])
-        cur.execute('INSERT INTO affiliated_org_urls VALUES(?, ?)',
-                    [id, filing['affiliated_orgs_url']])
-    return filing_rowid
 
 
-def _import_entity(record, cur, name, findrow, insert):
-    if name in record:
-        entity = record[name]
-        rowid = findrow(entity, cur)
-        if rowid is None:
-            return insert(entity, cur)
-        else:
-            return rowid
-    else:
-        return None
+_list_importers = {'lobbyist': _import_lobbyist,
+                   'govt_entity': _import_govt_entity,
+                   'issue': _import_issue,
+                   'org': _import_affiliated_org,
+                   'foreign_entity': _import_foreign_entity}
 
 
-def _import_registrant(record, cur):
-    return _import_entity(record,
-                         cur,
-                         'registrant',
-                          _registrant_rowid,
-                         _insert_registrant)
+def _import_list(entities, filing, cur):
+    """Import a list of parsed entities into the database.
 
+    Returns nothing.
 
-def _import_client(record, cur):
-    return _import_entity(record,
-                         cur,
-                         'client',
-                         _client_rowid,
-                         _insert_client)
+    Side-effects: inserts rows into the database.
+    
+    entitites - The list of parsed entities.
 
-
-def _import_lobbyist(record, cur):
-    return _import_entity(record,
-                         cur,
-                         'lobbyist',
-                          _lobbyist_rowid,
-                         _insert_lobbyist)
-
-
-def _import_govt_entity(record, cur):
-    return _insert_govt_entity(record['govt_entity'], cur)
-
-
-def _import_issue(record, cur):
-    return _insert_issue(record['issue'], cur)
-
-
-def _import_affiliated_org(record, cur):
-    return _import_entity(record,
-                         cur,
-                         'org',
-                         _affiliated_org_rowid,
-                         _insert_affiliated_org)
-
-
-def _import_list(record, id, entity_importer, cur):
-    ids = list()
-    if id in record:
-        for entity in record[id]:
-            ids.append(entity_importer(entity, cur))
-    return ids
-
-
-def _import_lobbyists(record, cur):
-    """Returns a list of rowids for the lobbyists in a filing record.
-
-    record - The parsed filing dictionary.
+    filing - The parsed filing dictionary with which the list is
+    associated.
 
     cur - The DB API 2.0-compliant database cursor.
 
     """
-    return _import_list(record, 'lobbyists', _import_lobbyist, cur)
+    def entity_id(entities):
+        # What kind of list is this? lobbyists? issues? etc.
+        return entities[0].keys()[0]
+    id = entity_id(entities)
+    importer = _list_importers[id]
+    for entity in entities:
+        importer(entity[id], filing, cur)
 
 
-def _import_govt_entities(record, cur):
-    """Returns a list of keys for the govt entities in a filing record.
+# Doesn't include an importer for 'filing'; that one is special.
 
-    record - The parsed filing dictionary.
-
-    cur - The DB API 2.0-compliant database cursor.
-
-    """
-    return _import_list(record, 'govt_entities', _import_govt_entity, cur)
-
-
-def _import_issues(record, cur):
-    """Returns a list of rowids for the issues in a filing record.
-
-    record - The parsed filing dictionary.
-
-    cur - The DB API 2.0-compliant database cursor.
-
-    """
-    return _import_list(record, 'issues', _import_issue, cur)
-
-
-def _import_affiliated_orgs(record, cur):
-    """Returns a list of rowids for the affiliated orgs in a filing record.
-
-    record - The parsed filing dictionary.
-
-    cur - The DB API 2.0-compliant database cursor.
-
-    """
-    return _import_list(record, 'affiliated_orgs', _import_affiliated_org, cur)
-
-
-_entity_importers = {'registrant': _import_registrant,
-                     'client': _import_client,
-                     'lobbyists': _import_lobbyists,
-                     'govt_entities': _import_govt_entities,
-                     'issues': _import_issues,
-                     'affiliated_orgs': _import_affiliated_orgs}
+_entity_importers = [('registrant', _import_registrant),
+                     ('client', _import_client),
+                     ('lobbyists', _import_list),
+                     ('govt_entities', _import_list),
+                     ('issues', _import_list),
+                     ('affiliated_orgs', _import_list),
+                     ('foreign_entities', _import_list)]
 
 
 def import_filings(cur, parsed_filings):
@@ -797,15 +839,16 @@ def import_filings(cur, parsed_filings):
 
     parsed_filings - A sequence of parsed filings.
 
-    Returns True.
+    Returns the cursor.
 
     """
     for record in parsed_filings:
         filing = record['filing']
-        for entity_name, entity_importer in _entity_importers.items():
-            filing[entity_name] = entity_importer(record, cur)
-        _insert_filing(filing, cur)
-    return True
+        _import_filing(filing, cur)
+        for entity_name, entity_importer in _entity_importers:
+            if entity_name in record:
+                entity_importer(record[entity_name], filing, cur)
+    return cur
 
 
 def create_db(con):
